#include <Adafruit_GFX.h>    // Core graphics library
#include <MCUFRIEND_kbv.h>   // Hardware-specific library
#include "AffichageTS35.h"  
MCUFRIEND_kbv tft;

// =================== Touch Screen constantes =================== //
// To change according to the calibration
// Valeurs à remplacer par celle données lors de la calibration
int XP=8,XM=A2,YP=A3,YM=9; //320x480 ID=0x9486
const int TS_LEFT=125,TS_RT=900,TS_TOP=942,TS_BOT=95;


#include "TouchScreen_kbv.h"         //hacked version
TouchScreen_kbv ts(XP, YP, XM, YM, 300);   //re-initialised after diagnose
TSPoint_kbv tp;                            //global point




// Fontes utilisées par le programme
#include <Fonts/FreeMonoBold12pt7b.h>
#include <Fonts/FreeSans12pt7b.h>
#include <Fonts/FreeMono9pt7b.h>
#include <FreeDefaultFonts.h>

// constante pour l'écran LARGEUR_ECRANxHAUTEUR_ECRAN -> affichage mesure

#define HauteurLigne1 8  // Hauteur 1ère ligne d'affichage
#define HauteurLigne2 21 // hauteur deuxième ligne d'affichage
#define LigneBouton 289  // ligne des boutons
#define AxePeriode 154
#define ColBouton 10
#define HautBouton 25

// fonctions internes
void readResistiveTouch(void);


// Variables globales pour l'affichage
// Affichage mesure
int iTrace;
int DerniereCol;
int DerniereTracePeriode1;
int DerniereTracePeriode2;
int DerniereTraceDiffTicTac;

/*
 * Init affichage écran
 */
void AfficheInit()
{
    uint16_t ID = tft.readID();
    if (ID == 0xD3) ID = 0x9481;
    tft.begin(ID);
    tft.setRotation(1);  
}

/*
 * Affiche un texte centré sur l'écran
 * ligne : ligne en pixel 
 * Couleur : couleur du texte
 * msg : texte à afficher
 */
void AfficheCentre(int ligne, const unsigned int Couleur, const char *msg) 
{
  int x1, y1, w, h, decal;
  tft.getTextBounds(msg, 0, 0,&x1, &y1, &w, &h);
  decal = (LARGEUR_ECRAN-w)/2;
  if (decal < 0) {decal = 0;}
  tft.setCursor(decal,ligne+h+10);
  tft.setTextColor(Couleur);
  tft.print(msg); 
}


/*
 * Affiche l'écran d'accueil
 */
void AfficheEcranAccueil()
{
  tft.setFont(&FreeMonoBold12pt7b);
  AfficheCentre(10,TFT_WHITE,"CHRONOCOMPARATEUR PC-RM3");
  tft.setFont(&FreeMono9pt7b);
  AfficheCentre(100,TFT_WHITE,"Version 2.1");
  AfficheCentre(130,TFT_WHITE,"http://www.pascalchour.fr");
}


/*
 * affiche l'écran de mesure
 * PBouton : L'appelant doit fournir un pointeur sur un tableau de structure de boutons valide (au moins un initialisée à -1, -1)
 */
void AfficheEcranBattement(const TBouton PBouton1[], const TBouton PBouton2[] )
{
  tft.fillScreen(CouleurFond);
  tft.setFont(&FreeSans12pt7b);
  AfficheCentre(10,TFT_WHITE,"Selection battement montre");
  AfficheBoutons(PBouton1);
  AfficheCentre(210,TFT_WHITE,"Selection occultation");
  AfficheBoutons(PBouton2);
}

/*
 * affiche l'écran de mesure
 * PBouton : L'appelant doit fournir un pointeur sur un tableau de structure de boutons contenant au moins un bouton valide
 * Seul le premier bouton de la liste est affiché
 */
void AfficheEcranMesure(const TBouton PBouton[])
{
    tft.fillScreen(CouleurFond);
    tft.drawFastVLine(0,1,HauteurLigne1+HauteurLigne2,TFT_WHITE);     
    tft.drawFastVLine(LargeurCol-1,1,HauteurLigne1+HauteurLigne2,TFT_WHITE);     
    tft.drawFastVLine(LargeurCol*2-1,1,HauteurLigne1+HauteurLigne2,TFT_WHITE);     
    tft.drawFastVLine(LargeurCol*3-1,1,HauteurLigne1+HauteurLigne2,TFT_WHITE);     
    tft.drawFastVLine(LargeurCol*4-1,1,HauteurLigne1+HauteurLigne2,TFT_WHITE);     
    tft.drawFastHLine(0,HauteurLigne1,LargeurCol*4-1,TFT_WHITE);
    tft.drawFastHLine(0,HauteurLigne1+HauteurLigne2,LargeurCol*4-1,TFT_WHITE);
    tft.drawFastHLine(0,AxePeriode,LargeurCol*4-1,TFT_WHITE);
    
    tft.drawFastHLine(0,AxePeriode-20,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.drawFastHLine(0,AxePeriode+20,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.drawFastHLine(0,AxePeriode-40,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.drawFastHLine(0,AxePeriode+40,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.drawFastHLine(0,AxePeriode-60,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.drawFastHLine(0,AxePeriode+60,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.drawFastHLine(0,AxePeriode-80,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.drawFastHLine(0,AxePeriode+80,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.drawFastHLine(0,AxePeriode-100,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.drawFastHLine(0,AxePeriode+100,LargeurCol*4-1,TFT_LIGHTGREY);
    tft.setTextSize(1);
    AfficheBoutons(PBouton);  
    AfficheBloc(1,CouleurFond,TFT_GREEN,"Diff TicTac");
    AfficheBloc(2,CouleurFond,TFT_GREEN,"Diff periode");
    AfficheBloc(3,CouleurFond,TFT_GREEN,"Moy periode");
    AfficheBloc(4,CouleurFond,TFT_GREEN,"A/R jour");
    iTrace = 0; // position courante du tracé
    DerniereTracePeriode1 = 0; // position précédente du tracé pour la différence entre TicTac mesuré et période attendue
    DerniereTracePeriode2 = 0; // position précédente du tracé pour la différence entre TicTac mesuré et période attendue
    DerniereTraceDiffTicTac = 0; // position précédente du tracé pour la différence entre Tic et Tac
    DerniereCol = 0; 
}

/*
 * Affiche un texte au n° du bloc donné en paramètre
 * Efface le bloc au préalable (rempli un rectangle avec la couleur de fond couleurFondd)
 * bloc 1 à 4 -> 4 premiers blocs (1ère ligne) centrés.
 * bloc 5 à 8 -> 4 blocs suivants (2ème ligne) à droite.
 */
void AfficheBloc(int bloc, unsigned int couleurFond, unsigned int couleurCar, const char *msg)
{
  int x1, y1, w, h, decal,i;
// Calculer largeur texte à afficher (à faire) 
  if (bloc < 9) {
    tft.setTextColor(couleurCar);
    if ((bloc > 0) && (bloc < 5)) { // On centre les textes de 1 à 4
      tft.setFont(NULL);
      tft.getTextBounds(msg, 0, 0,&x1, &y1, &w, &h);
      decal = LargeurCol-w;
      if (decal < 0) {decal = 0;}
      decal = (decal / 2)+(bloc-1)*LargeurCol+bloc;
      tft.setCursor(decal, 0);
      tft.print(msg);    
    } else {
      bloc = bloc-5;
      tft.setFont(&FreeMonoBold12pt7b);
      tft.getTextBounds(msg, 0, 0,&x1, &y1, &w, &h);
      decal = LargeurCol-w-5+bloc*LargeurCol;
      if (decal < 0) {decal = 0;}
      tft.fillRect(bloc*LargeurCol+1, HauteurLigne1+HauteurLigne2-20, LargeurCol-2, HauteurLigne2-2, couleurFond);
      tft.setCursor(decal, HauteurLigne1+HauteurLigne2-2);
      tft.print(msg);    
    }
  }
}

/*
 * Affiche les boutons de la structure passée en paramètre
 */

void AfficheBoutons(const TBouton PBouton[])
{
  int i;
  i = 0;
  while (PBouton[i].ligne != -1) {
    AfficheUnBouton(&PBouton[i],false);
    i = i+1;
  }
}

/*
 * Affiche un bouton dont les coordonnées sont données en paramètre = PBouton
 * Affiche le texte centré dans le bouton si c'est possible
 * Selection = vrai : bouton sélectionné, = faux, bouton normal
 */
void AfficheUnBouton(const TBouton *PBouton,  bool Selection)
{
  int x1, y1, w, h, decal,i;
  unsigned int ColFond, ColCar;

  if (Selection) {
    ColFond = PBouton->couleurFondSel;
    ColCar = PBouton->couleurCarSel;
  } else {
    ColFond = PBouton->couleurFond;
    ColCar = PBouton->couleurCar;
  }
  tft.setFont(&FreeSans12pt7b);
  i = 0;
  tft.getTextBounds(PBouton->buf, 0, 0,&x1, &y1, &w, &h);
  decal = PBouton->largeur-w;
  if (decal < 0) {decal = 0;}
  decal = decal / 2;
  tft.fillRoundRect(PBouton->colonne, PBouton->ligne, PBouton->largeur,HautBouton,3,ColFond);
  tft.setCursor(PBouton->colonne+decal,PBouton->ligne+HautBouton-5);
  tft.setTextColor(ColCar);
  tft.print(PBouton->buf);
}

/*
 * Affiche le bouton passé en paramètre et fait basculer les couleurs entre bouton sélectionné et bouton non sélectionné 
 * pour donner une impression de clignotement
 */
void ClignoteBouton(const TBouton *PBouton)
{
  int i;
  for (i=0; i<5; i++) {
    AfficheUnBouton(PBouton, true);
    delay(20);
    AfficheUnBouton(PBouton, false);
    delay(20);
  }
}


/*
 * Donne l'index du n° du bouton pressé dans le tableau de structure donné en paramètre
 * Les coordonnées du dernier bouton doivent valoir -1 pour arrêter la recherche dans le tableau
 * La fonction renvoie -1 si aucune coordonnées de bouton correspond aux coordonnées où il y a eu appuie sur l'écran
 */

int NumBouton(const TBouton PBouton[])
{
  long x;
  long y;
  int i = 0;
// Attention : les coordonnées de l'écran dépendent de son orientation
// Dans le cas présent, les coordonnées "lignes" sont inversées d'où HAUTEUR_ECRAN, 0 au lieu de 0, HAUTEUR_ECRAN. 
// Faire des essais si problème et afficher les coordonnées en retirant le commentaire
// sur la ligne serial.println
  x = map(tp.x, TS_LEFT, TS_RT, HAUTEUR_ECRAN, 0);
  y = map(tp.y, TS_TOP, TS_BOT, 0, LARGEUR_ECRAN);
// Serial.println("x=" + String(x) + ", y=" + String(y) + ", tp.z =" + String(tp.z));  
  while (PBouton[i].ligne != -1) {
    if ((x >= PBouton[i].ligne) && (x < PBouton[i].ligne+HautBouton)) {
      if ((y >= PBouton[i].colonne) && (y <= PBouton[i].colonne+PBouton[i].largeur)) {
        return i;
      }
    }
    i = i+1;
  }
  return -1;
}

/*
 * Affiche la trace de la mesure
 * val1 = différence entre demi période attendue et période mesurée pour Tic
 * val2 = différence entre demi période attendue et période mesurée pour Tac
 * val3 = différence entre Tic et Tac
 */
void AfficheTrace(long val1,long val2)
{
  if (iTrace >= tft.width()) {
    tft.drawFastHLine(0,AxePeriode,LargeurCol*4-1,TFT_WHITE);
    iTrace = 0;
    DerniereCol = 0;
  }
  tft.drawFastVLine(iTrace,AxePeriode-120,242,CouleurFond); // On efface la trace du passage précédent
  tft.drawPixel(iTrace,AxePeriode,TFT_RED); // et l'on reconstitue les lignes effacées sauf celle du milieu que l'on marque en rouge
  tft.drawPixel(iTrace,AxePeriode-20,TFT_LIGHTGREY);
  tft.drawPixel(iTrace,AxePeriode+20,TFT_LIGHTGREY);
  tft.drawPixel(iTrace,AxePeriode-40,TFT_LIGHTGREY);
  tft.drawPixel(iTrace,AxePeriode+40,TFT_LIGHTGREY);
  tft.drawPixel(iTrace,AxePeriode-60,TFT_LIGHTGREY);
  tft.drawPixel(iTrace,AxePeriode+60,TFT_LIGHTGREY);
  tft.drawPixel(iTrace,AxePeriode-80,TFT_LIGHTGREY);
  tft.drawPixel(iTrace,AxePeriode+80,TFT_LIGHTGREY);
  tft.drawPixel(iTrace,AxePeriode-100,TFT_LIGHTGREY);
  tft.drawPixel(iTrace,AxePeriode+100,TFT_LIGHTGREY);

  if ((val1 >= -120) && (val1 <= 110))   {tft.drawPixel(iTrace,AxePeriode-val1,TFT_YELLOW);}
  if ((val2 >= -120) && (val2 <= 110))   {tft.drawPixel(iTrace,AxePeriode-val2,TFT_YELLOW);}
  
  DerniereCol = iTrace;
  DerniereTracePeriode1 = AxePeriode-val1;
  DerniereTracePeriode2 = AxePeriode-val2;
  iTrace = iTrace+1;
}



/*
 * Indique si on a appuyé sur l'écran
 */
 
bool IsPressed(void)
{
    // .kbv this was too sensitive !!
    // now touch has to be stable for 50ms
    int count = 0;
    bool state, oldstate;
    while (count < 10) {
        readResistiveTouch();
        state = tp.z > 200;     //ADJUST THIS VALUE TO SUIT YOUR SCREEN e.g. 20 ... 250
        if (state == oldstate) count++;
        else count = 0;
        oldstate = state;
        delay(5);
    }
    return oldstate;
}

/*
 * Lit les valeurs d'un appui sur l'écran (X, Y, Z)
 */
void readResistiveTouch(void)
{
    tp = ts.getPoint();
    pinMode(YP, OUTPUT);      //restore shared pins
    pinMode(XM, OUTPUT);
    digitalWrite(YP, HIGH);  //because TFT control pins
    digitalWrite(XM, HIGH);
    //    Serial.println("tp.x=" + String(tp.x) + ", tp.y=" + String(tp.y) + ", tp.z =" + String(tp.z));
}
